%{
%}

%token <string> IDENTIFIER
%token <int32> INT_CONST NEG_INT_CONST
%token <string> QUOTED_STRING

%token EOF FUNCTION_DIRECTIVE TEST_DIRECTIVE COLON LBRACE RBRACE SEMI
%token PARAMETERS_DIRECTIVE IN_DIRECTIVE OUT_DIRECTIVE INOUT_DIRECTIVE
%token INT_SPECIFICATION TUPLE_SPECIFICATION STRING_SPECIFICATION
%token LBRACKET RBRACKET EQUALS GLOBVARS_DIRECTIVE
%token TEST_DIRECTIVE INPUTS_DIRECTIVE LPAREN RPAREN COMA RESULTS_DIRECTIVE


%start file
%type <Basics.shot> file
%%

file:
    sections EOF  { $1 }
;


sections:
    description_section parameters_section globvars_section scenario_list
    {
     { Basics.tested_function = $1 ;
       Basics.tested_parameters = $2 ;
       Basics.tested_globvars  = $3 ;
       Basics.tested_tests = $4 }
   }
;

description_section:
    FUNCTION_DIRECTIVE LBRACE IDENTIFIER RBRACE              { $3 }
;

parameters_section:
    PARAMETERS_DIRECTIVE LBRACE declaration_list RBRACE      { $3 }
;

globvars_section:
    GLOBVARS_DIRECTIVE LBRACE declaration_list RBRACE        { $3 }
;

declaration_list:
                                    { [] }
  | declaration declaration_list    { $1 :: $2 }
;

/* Utilis pour dclarer les variables globales et les paramtres. */
declaration:
  mode_specifier IDENTIFIER COLON type_specifier SEMI
  {
   { Basics.var_mutability = $1 ;
     Basics.var_name = $2 ;
     Basics.var_type = fst $4 ;
     Basics.var_init = snd $4 }
 }
;

mode_specifier:
    IN_DIRECTIVE     { Basics.IN }
  | OUT_DIRECTIVE    { Basics.OUT }
  | INOUT_DIRECTIVE  { Basics.INOUT }
;

type_specifier:
    INT_SPECIFICATION int_opt_init                  { (Basics.INT , $2)}
  | INT_SPECIFICATION LBRACKET INT_CONST RBRACKET int_opt_init
      {
       (* Bon, on ne va quand mme pas faire des tableaux de 2^16 entres ! *)
       ((Basics.INT_ARRAY (Int32.to_int $3)), $5)
      }
  | STRING_SPECIFICATION string_opt_init            { (Basics.STRING, $2) }
  | TUPLE_SPECIFICATION  tuple_opt_init             { (Basics.TUPLE, $2) }
;

int_opt_init:
	                { None }
  | EQUALS valnum       { (Some (Basics.Init_integer $2)) }
;

string_opt_init:
	                        { None }
  | EQUALS QUOTED_STRING        { Some (Basics.Init_string $2) }
;

tuple_opt_init:
	                        { None }
  | EQUALS tuple_format         { Some $2 }
;

scenario_list:
                                { [] }
  | scenario scenario_list      { $1 :: $2 }
;

scenario:
    TEST_DIRECTIVE INT_CONST QUOTED_STRING LBRACE inputs results RBRACE
      {
       { Basics.test_number = (Int32.to_int $2) ;
	 Basics.test_comment = $3 ;
	 Basics.test_inputs = $5 ;
	 Basics.test_results = $6 }
     }
;

inputs:
   INPUTS_DIRECTIVE LBRACE ident_bindings RBRACE   { $3 }
;

results:
   RESULTS_DIRECTIVE LBRACE ident_bindings RBRACE   { $3 }
;

ident_bindings:
                                         { [] }
  | binding ident_bindings               { $1 :: $2 }
;

tuple_format:
  | LPAREN INT_CONST COMA INT_CONST RPAREN {(Basics.Init_tuple ($2,$4))}
;

binding:
    LPAREN IDENTIFIER COMA valnum RPAREN SEMI
      {
       let variable = Basics.Var_atomic $2 in
       (variable, (Basics.Init_integer $4))
     }
  | LPAREN IDENTIFIER LBRACKET INT_CONST RBRACKET COMA valnum RPAREN SEMI
      {
       let variable = Basics.Var_array ($2, (Int32.to_int $4)) in
       (variable, (Basics.Init_integer $7))
     }
  | LPAREN IDENTIFIER COMA QUOTED_STRING RPAREN SEMI
      {
       let variable = Basics.Var_atomic $2 in
       (variable, (Basics.Init_string $4))
     }
  | LPAREN IDENTIFIER COMA tuple_format RPAREN SEMI
      {
       let variable = Basics.Var_atomic $2 in
       (variable, $4)
     }
;

valnum:
    INT_CONST     { $1 }
  | NEG_INT_CONST { Int32.neg $1 }
;
